
/* Upravil a doplnil funkcie pablox, www.pablox.net, pablox@pablox.net 
 *  10/2023 pridana moznost nastavenia casu zapnutia a vypnutia podsvietenia LCD cez konfiguracne prikazy
 *  10/2023 doplnene meranie UV ziarenia snimacom VEML6075
 *  10/2023 upravene nacitanie casu cez NTP, automaticke prepinanie letny/zimny cas
 *  7/2023 doplnene funkcie, cast prevzata z povodnej kontrukcie  
 *  https://www.pablox.net/index.php?option=com_content&view=article&id=138:detektor-brok-s-obvodom-as3935&catid=20:indiktoy&Itemid=36
 *   
 *   ! Mnou upraveny zdrojovy kod a zapojenie na povodnej stranke nesedi, nutne pouzit mnou upravene zapojenie !
 *   
 *  Kompilovane pod Arduino IDE 1.8.18
 *  Doplnenie znakoveho LCD 20x4 s radicom HD44780 s i2c komunikaciou
 *  Meranie UV ziarenia snimacom VEML6075
 *  Meranie vlhkosti cidlom HTU21D
 *  Meranie teploty a vlhkosti cidlom BMP180
 *  Meranie intenzity vonkajsieho osvetlenia cidlom BH1750
 *  Nacitanie casu z NTP servera a synchronizacia interneho RTC
 *  moznost odosielania udajov na server v 15min intervaloch (nutne si doprogramovat cast na strane servera)
 *  moznost odosielania udajov o zachytenych bleskoch na sever (nutne si doprogramovat cast na strane servera)
 *  moznost aktualizacie FW cez OTA a wifi
 *  
 *  Pred kompilaciou upravit nastavenia v myWIHI.h - MY_SSID, MY_PSK, MY_APIKEY
 *  
 * Originalny zdrojak:
 * Kaminari
 *
 * Copyright (C) 2020 Richard "Shred" Körber
 *   https://github.com/shred/kaminari
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <SPI.h>
#include <ArduinoJson.h>
#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
#include <Adafruit_NeoPixel.h>
#include <PubSubClient.h>
#include <ESP8266HTTPClient.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h> // LCD 20x4 cez i2c s PCF8574
#include <SFE_BMP180.h> // BMP180
#include <HTU21D.h> // HTU21 
#include <BH1750.h> // BH1750
#include <VEML6075.h> // VEML6075 UV snimac
//OTA
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
//NTP pre odosielanie udajov na server v uvedeny cas
#include <time.h> 

#include "AS3935.h"
#include "Config.h"
#include "myWiFi.h"

// NUTNE NASTAVIT UZIVATELOM

#define alt 475 // nadmorska vyska v ktorej je detektor - nutne pre spravne zobrazenie atmosferickeho tlaku

// nastavenie casoveho servera, nastavenie prechodu na zimny/letny cas, viac v odkaze
//Letny cas sa meni treti mesiac (M3), piaty tyzden (5), v nedelu (0) o 2 hodine rano (02). 
//Zimny cas sa meni 10ty mesiac (M10), piaty tyzden (5), v nedelu (0) o tretej hodine (03).
// https://werner.rothschopf.net/202011_arduino_esp8266_ntp_en.htm
#define MY_NTP_SERVER "pool.ntp.org"           
#define MY_TZ "CET-1CEST,M3.5.0/02,M10.5.0/03"   

// pripojenie AS3935
#define SPI_CS   15     // IO15 (D8) AS3935
#define SPI_MOSI 13     // IO13 (D7) AS3935
#define SPI_MISO 12     // IO12 (D6) AS3935
#define SPI_SCK  14     // IO14 (D5) AS3935
#define AS_INT    5     // IO5  (D1) AS3935

#define NEOPIXEL  2     // IO4  (D2) SK6812 RGBW LED

#define PORT 80

#define COLOR_CONNECTING   0x00800000
#define COLOR_CALIBRATING  0x00404000
#define COLOR_BLACK        0x00000000

#define DISTANCE_ANIMATION_TIME 5000
#define ENERGY_ANIMATION_TIME    500
#define DISTURBER_ANIMATION_TIME 1000

ConfigManager cfgMgr;
ESP8266WebServer server(PORT);
AS3935 detector(SPI_CS, AS_INT);
Adafruit_NeoPixel neopixel(1, NEOPIXEL, NEO_GRBW + NEO_KHZ800);
WiFiClient wifiClient;
PubSubClient client(MY_MQTT_SERVER_HOST, MY_MQTT_SERVER_PORT, wifiClient);
WiFiEventHandler connectedEventHandler, gotIpEventHandler, disconnectedEventHandler;

unsigned long beforeAnimation = millis();
unsigned long beforeMqttConnection = millis();
bool connected = false;
unsigned int currentColor = 0;
Lightning ledLightning;

// snimanie velicin z cidiel
double T,P; // teplota, tlak
double p0,a; // tlak vztiahnuty na hladinu mora
double humid; // vlhkost
float fT; // teplota vo float
int iH, iP, iL; // ostatne merania v int
uint16_t lux; // osvetlenie v lux

long previousMillis = 0; // cas posledneho merania a aktualizacie LCD
long LCDupdate_interval = 10000; // interval aktualizacie LCD a dat zo snimacov
bool snimaceOdoslane = false; // aby sa zapis odoslal len raz 

//zobrazenie casu od posledneho blesku na LCD
int LCDh, LCDm = 0;

SFE_BMP180 pressure;
HTU21D humidity;
BH1750 light;
VEML6075 veml6075 = VEML6075();

// premenne pre ulozenie UV merani
float UVA; 
float UVB; 
float UVI; // UV index 

// SPRACOVANIE BLESKOV 
int b_distance, b_energy, b_intensity = 0;
unsigned long lastLightningTime = 0;
Lightning LCDlightning;

// premenne pre zobrazovanie nad LCD
char LCDline1[20];
char LCDline2[20];
char LCDline3[20];
char LCDline4[20];
bool LCDtext = 0; // prepinanie zobrazenia udajov na spodnom riadku LCD 
LiquidCrystal_I2C lcd(0x20,20,4);  // LCD na adrese 0x27 (0x20 adresne vstupy A0-A2 uzemnene, 0x27 - adresne vstupy v log 1)

// specialne znaky pre LCD
uint8_t znakTeplomer[8] = { 0b00100,   0b01010,  0b01010,  0b01110,  0b11111,  0b11111,  0b01110,  0b00000 };
uint8_t znakHodiny[8] = { 0b00000,  0b01110,  0b10101,  0b10111,  0b10001,  0b01110,  0b00000,  0b00000 };
uint8_t znakVlhkost[8] = {  0b00000,0b00100,0b01110,0b11111,0b11111,0b11111,0b01110,0b00000 };
uint8_t znakTlak[8] = {   0b00100,0b00100,0b00100,0b00100,0b00100,0b11111,0b01110,0b00100 };
uint8_t znakOsvetlenie[8] = {  0b00000,0b01110,0b10001,0b10001,0b10001,0b01110,0b01110,0b00100 };
uint8_t znakBlesk[8] = { 0b00001,0b00010,0b00100,0b01110,0b00010,0b10100,0b11000,0b11100 };
uint8_t znakSum[8] = { 0b00000,0b11111,0b10001,0b01000,0b00100,0b01000,0b10001,0b11111 };
uint8_t znakIntenzita[8] = { 0b00000,0b00001,0b00001,0b00101,0b00101,0b10101,0b10101,0b10101 };

//casove premenne
time_t now;                         // sekundy od roku 1970 (1970) - UTC
tm tm;                              // objekt pre normalnejsie ulozenie casovych premennych 
    
void setup() {
    Serial.begin(115200);
    Serial.println("");

    configTime(MY_TZ, MY_NTP_SERVER); // konfiguracia casu
    
    Wire.setClock(400000L);
    Wire.begin(4, 0); //i2C na inych pinoch ako je bezne (SDA, SCL) 

    lcd.init(); // inicializacia LCD
    lcd.clear(); 
    lcd.backlight();   // zapne podsvietenie LCD 
   
    lcd.createChar(0, znakHodiny);
    lcd.createChar(1, znakIntenzita);
    lcd.createChar(2, znakTeplomer);
    lcd.createChar(3, znakVlhkost);
    lcd.createChar(4, znakTlak);
    lcd.createChar(5, znakOsvetlenie);
    lcd.createChar(6, znakBlesk);
    lcd.createChar(7, znakSum);
    
    // nacita intenzitu osvetlenia
    light.begin();
    //nacita vlhkost
    humidity.begin();
   
// BMP180 inicializacia - stav vypisuje na UART
  if (pressure.begin())
    {
    Serial.println(F("BMP180 init OK"));
    }
  else
    {
  // BMP180 chyba
    Serial.println(F("BMP180 init fail"));
    }
  // koniec inicializacie BMP180

 // VEML6075 inicializacia - stav vypisuje na UART
  if (!veml6075.begin()) {
    Serial.println(F("VEML6075 init FAIL"));
  } else {
    Serial.println(F("VEML6075 init OK"));
  }
// koniec inicializacie VEML6075

    cfgMgr.begin();
    detector.begin();

    neopixel.begin();
    neopixel.setBrightness(255);

    // Init detector
    color(COLOR_CALIBRATING);
    Serial.println("Calibrating antenna...");
    detector.reset();
    unsigned int freq = detector.calibrate();
    Serial.print("Calibrated antenna frequency: ");
    Serial.print(freq);
    Serial.println(" Hz");
    
    lcd.setCursor(0,0);
    lcd.print("Osc f= ");
    lcd.print(freq);
    lcd.print("Hz"); 
    delay(2000);
    color(COLOR_BLACK);

    // Set up the detector
    setupDetector();

    // Start WiFi
    WiFi.setSleepMode(WIFI_NONE_SLEEP);
    WiFi.mode(WIFI_STA);
    connectedEventHandler = WiFi.onStationModeConnected([](const WiFiEventStationModeConnected& event) {
        lcd.clear();
        Serial.print("WiFi connected to: ");
        Serial.println(WiFi.SSID());
        lcd.setCursor(0,1);
        lcd.print(WiFi.SSID());
        Serial.print("MAC: ");
        lcd.setCursor(0,2);
        lcd.print(WiFi.macAddress());
        Serial.println(WiFi.macAddress());
    });
    gotIpEventHandler = WiFi.onStationModeGotIP([](const WiFiEventStationModeGotIP& event) {
        Serial.print("IP address: ");
        Serial.println(WiFi.localIP());
        lcd.setCursor(0,3);
        lcd.print(WiFi.localIP());
        server.begin();
        Serial.print("Server is listening on port: ");
        Serial.println(PORT);
        connected = true;
    });
    disconnectedEventHandler = WiFi.onStationModeDisconnected([](const WiFiEventStationModeDisconnected& event) {
        lcd.clear();
        connected = false;
        Serial.println("WiFi connection lost. Reconnecting...");
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("Wifi con err");
        WiFi.begin(MY_SSID, MY_PSK);
    });
    WiFi.begin(MY_SSID, MY_PSK);

    // Start MDNS
    if (MDNS.begin(MY_MDNS_NAME)) {
        Serial.println("MDNS responder started");
        MDNS.addService("http", "tcp", PORT);
    }

    // Start server
    server.on("/status", handleStatus);
    server.on("/settings", handleSettings);
    server.on("/update", handleUpdate);
    server.on("/calibrate", handleCalibrate);
    server.on("/clear", handleClear);
    server.on("/reset", handleReset);
    server.onNotFound([]() {
    server.send(404, "text/plain", server.uri() + ": not found\n");
    });
    const char * headerkeys[] = { "X-API-Key" };
    server.collectHeaders(headerkeys, sizeof(headerkeys) / sizeof(char*));
    delay(5000);


// OTA pre aktualizaciu FW cez wifi
  ArduinoOTA.onStart([]() {
    String type;
    if (ArduinoOTA.getCommand() == U_FLASH) {
      type = "sketch";
    } else {  // U_FS
      type = "filesystem";
    }

    // NOTE: if updating FS this would be the place to unmount FS using FS.end()
    Serial.println("Start updating " + type);
  });
  ArduinoOTA.onEnd([]() {
    Serial.println("\nEnd");
  });
  ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
    Serial.printf("Progress: %u%%\r", (progress / (total / 100)));
  });
  ArduinoOTA.onError([](ota_error_t error) {
    Serial.printf("Error[%u]: ", error);
    if (error == OTA_AUTH_ERROR) {
      Serial.println("Auth Failed");
    } else if (error == OTA_BEGIN_ERROR) {
      Serial.println("Begin Failed");
    } else if (error == OTA_CONNECT_ERROR) {
      Serial.println("Connect Failed");
    } else if (error == OTA_RECEIVE_ERROR) {
      Serial.println("Receive Failed");
    } else if (error == OTA_END_ERROR) {
      Serial.println("End Failed");
    }
  });
  ArduinoOTA.begin();
// koniec OTA
    
    lcd.clear();
    sensors_getdata();  // prvotne nacitanie udajov zo snimacov
}


// ================ HLAVNA SLUCKA =================
void loop() {
    ArduinoOTA.handle();  // obsluha OTA
    
    //ak treba, odkomentovat pre vypis nameranych hodnot zo senzora UV 
   /* Serial.print("UVA: ");
    Serial.print(String(UVA));
    Serial.print(" , UVB: ");
    Serial.print(String(UVB));
    Serial.print(" , UVI: ");
    Serial.println(String(UVI));*/
    
    time(&now);                       // nacita aktualny cas
    localtime_r(&now, &tm);           // aktualizuje strukturu s casovymi udajmi
    unsigned long currentMillis = millis();
    unsigned long currentMillisSnimace = millis();
    const unsigned long now = millis();
    unsigned long lastLightningTime_min;
    lastLightningTime_min = timeDifference(now, lastLightningTime) / 60000; // cas od poslednej udalosti v minutach

    LCDh = lastLightningTime_min / 60; // prevod casu od posledneho blesku na hodiny
    LCDm = lastLightningTime_min - (LCDh * 60); // prevod casu od posledneho blesku na minuty

// OBNOVUJE LCD V NASTAVENOM INTERVALE
  if(currentMillis - previousMillis > LCDupdate_interval) {
    LCDbacklightControl(); // ovlada podsvietenie LCD, vypnute od 22:00 do 6:00
    lcd.clear();
    lcd.setCursor(0,0);
    sprintf(LCDline1, "\002 %.1f C  \003 %d%%", fT, iH);
    lcd.print(LCDline1);
    lcd.setCursor(0,1);
    sprintf(LCDline2, "\004 %dmBar \005 %dlx" , iP, iL);
    lcd.print(LCDline2);
    lcd.setCursor(0,2);
    sprintf(LCDline3, "Last \006 %dkm  \001 %d" , b_distance, b_intensity);
    lcd.print(LCDline3);
    
    // prepina zobrazenie na spodnom riadku LCD po kazdom jeho obnoveni : informacie o poslednom blesku / aktualny UV index 
    if (LCDtext == 0) 
      { 
        lcd.setCursor(0,3);
        lcd.write(0);
        lcd.setCursor(1,3);
        sprintf(LCDline4, " %dh%dmin  \007\006 %d" , LCDh, LCDm , detector.strikesTotal);
        LCDtext = 1;
      } 
     else 
      { 
        lcd.setCursor(0,3);
        sprintf(LCDline4, "UV index: %.1f" , UVI);
        //sprintf(LCDline4, "UV index: %d" , UVI);
        LCDtext = 0;
      }    
    lcd.print(LCDline4);
    
    previousMillis = currentMillis;      
  }
// KONIEC

// ODOSIELA UDAJE ZO SNIMACOV NA SERVER V 15 MINUTOVYCH INTERVALOCH
  
if ((tm.tm_min %15 != 0) && (snimaceOdoslane == 1)) 
    { 
     snimaceOdoslane = 0; 
    }

  if ((tm.tm_min %15 == 0) && (snimaceOdoslane == 0)) 
    { 
      Serial.println("Ubehlo 15minut");
      sensors_getdata();
      write_snimace(); // funkcia zabezpecujuca odoslanie na server cez PHP
     
      Serial.println("NOVE MERANIE");
      Serial.print("Teplota ");
      Serial.println(String(fT));
      Serial.print("Vlhkost ");
      Serial.println(String(iH));
      Serial.print("Tlak ");
      Serial.println(String(iP));
      Serial.print("Osvetlenie ");
      Serial.println(String(iL));
     
    snimaceOdoslane = 1; 
    }
// KONIEC
      
//ZISTUJE, CI JE ZAZNAMENANY NOVY BLESK A NASLEDNE SPRACUJE UDAJE
    if (detector.update()) 
    {
       // cas posledneho blesku
       if (detector.getLastLightningDetection(0, LCDlightning)) {
            lastLightningTime = LCDlightning.time;
        }
      
      if (detector.isNewStrike == 1) 
        {
          Serial.print("Blesk... ");
          Serial.print("spolu ");
          Serial.println(String(detector.strikesTotal));
                          
          Lightning lightning;
          if (detector.getLastLightningDetection(0, lightning)) {
            b_energy = lightning.energy;
          if (lightning.distance < 0x3F) {
            b_distance = lightning.distance;
          } else {
            b_distance = 0;
          }
          } else  {
            b_energy = 0;
            b_distance = 0;
            }
          detector.isNewStrike = 0;
          if (detector.isNewStrike == 0)
          {
            Serial.println("Posledny blesk zmazany");  
          }
          b_intensity = map(b_energy,1,1000000,1,10); 
          write_blesk();
        }
     }
// koniec
        
    if (connected) {
        server.handleClient();
        MDNS.update();

        #ifdef MY_MQTT_ENABLED
        if (!client.loop() && timeDifference(now, beforeMqttConnection) > 1000) {
            beforeMqttConnection = now;
            if (client.connect(MY_MQTT_CLIENT_ID, MY_MQTT_USER, MY_MQTT_PASSWORD)) {
                Serial.println("Successfully connected to MQTT server");
            } else {
                Serial.print("Connection to MQTT server failed, rc=");
                Serial.println(client.state());
            }
        }
        #endif
    }

    if (detector.update()) {
        if (!detector.getLastLightningDetection(0, ledLightning)) {
            ledLightning.time = 0;
        }
        #ifdef MY_MQTT_ENABLED
        if (connected && client.connected()) {
            sendMqttStatus();
        }
        #endif
    }

    if (timeDifference(now, beforeAnimation) > 50) {
        beforeAnimation = now;
        updateColor(now);
    }
}



inline static unsigned long timeDifference(unsigned long now, unsigned long past) {
    // This is actually safe from millis() overflow because all types are unsigned long!
    return past > 0 ? (now - past) : 0;
}

void sendJsonResponse(ArduinoJson::JsonDocument &doc, int status = 200) {
    String json;
    serializeJson(doc, json);
    server.send(status, "application/json", json);
}

bool authenticated() {
    if (server.header("X-API-Key") == MY_APIKEY) {
        return true;
    }
    if (server.arg("api_key") == MY_APIKEY) {
        return true;
    }
    server.send(403);
    return false;
}

void handleStatus() {
    DynamicJsonDocument doc(8192);

    // sformatuje datum a cas aby sa dal vypisat
    char jsontime[14];
    char jsondate[14];
    sprintf(jsontime, "%d:%d:%d", tm.tm_hour, tm.tm_min, tm.tm_sec);
    sprintf(jsondate, "%d-%d-%d", tm.tm_mday, tm.tm_mon + 1, tm.tm_year + 1900);
    
    unsigned long now = millis();

    JsonArray la = doc.createNestedArray("lightnings");
    Lightning lightning;
    for (int ix = 0; ; ix++) {
        if (!detector.getLastLightningDetection(ix, lightning)) {
            break;
        }
        StaticJsonDocument<200> subdoc;
        subdoc["age"] = timeDifference(now, lightning.time) / 1000;
        subdoc["energy"] = lightning.energy;
        if (lightning.distance < 0x3F) {
            subdoc["distance"] = lightning.distance;
        } else {
            subdoc["distance"] = (char*) NULL;
        }
        la.add(subdoc);
    }

    unsigned int distance = detector.getEstimatedDistance();
    if (distance < 0x3F) {
        doc["distance"] = distance;
    } else {
        doc["distance"] = (char*) NULL;
    }

    doc["energy"] = detector.getEnergy();
    doc["noiseFloorLevel"] = detector.getNoiseFloorLevel();
    doc["disturbersPerMinute"] = detector.getDisturbersPerMinute();
    doc["watchdogThreshold"] = detector.getWatchdogThreshold();
    doc["wifiSignalStrength"] = WiFi.RSSI();
    doc["temperature"] = fT;
    doc["humidity"] = iH;
    doc["pressure"] = iP;
    doc["lightIntensity"] = iL;
    doc["UVAlevel"] = UVA;
    doc["UVBlevel"] = UVB;
    doc["UVindex"] = UVI;
    doc["strikesTotal"] = detector.strikesTotal;
    doc["time"] = jsontime;
    doc["date"] = jsondate;
    sendJsonResponse(doc);
}

void handleSettings() {
    DynamicJsonDocument doc(1024);
    doc["tuning"] = detector.getFrequency();
    doc["noiseFloorLevel"] = detector.getNoiseFloorLevel();
    doc["outdoorMode"] = detector.getOutdoorMode();
    doc["watchdogThreshold"] = detector.getWatchdogThreshold();
    doc["minimumNumberOfLightning"] = detector.getMinimumNumberOfLightning();
    doc["spikeRejection"] = detector.getSpikeRejection();
    doc["outdoorMode"] = detector.getOutdoorMode();
    doc["statusLed"] = cfgMgr.config.ledEnabled;
    doc["blueBrightness"] = cfgMgr.config.blueBrightness;
    doc["disturberBrightness"] = cfgMgr.config.disturberBrightness;
    doc["lcdONhour"] = cfgMgr.config.lcdON;
    doc["lcdOFFhour"] = cfgMgr.config.lcdOFF;
    sendJsonResponse(doc);
}

void handleUpdate() {
    if (authenticated()) {
        bool needsClearing = false;

        if (server.hasArg("watchdogThreshold")) {
            long val = String(server.arg("watchdogThreshold")).toInt();
            if (val >= 0 && val <= 10) {
                cfgMgr.config.watchdogThreshold = val;
                detector.setWatchdogThreshold(val);
                needsClearing = true;
            }
        }

        if (server.hasArg("minimumNumberOfLightning")) {
            long val = String(server.arg("minimumNumberOfLightning")).toInt();
            if (val == 1 || val == 5 || val == 9 || val == 16) {
                cfgMgr.config.minimumNumberOfLightning = val;
                detector.setMinimumNumberOfLightning(val);
            }
        }

        if (server.hasArg("spikeRejection")) {
            long val = String(server.arg("spikeRejection")).toInt();
            if (val >= 0 && val <= 11) {
                cfgMgr.config.spikeRejection = val;
                detector.setSpikeRejection(val);
                needsClearing = true;
            }
        }

        if (server.hasArg("outdoorMode")) {
            bool val = String(server.arg("outdoorMode")) == "true";
            cfgMgr.config.outdoorMode = val;
            detector.setOutdoorMode(val);
            needsClearing = true;
        }

        if (server.hasArg("statusLed")) {
            cfgMgr.config.ledEnabled = String(server.arg("statusLed")) == "true";
        }

        if (server.hasArg("blueBrightness")) {
            long val = String(server.arg("blueBrightness")).toInt();
            if (val >= 0 && val <= 255) {
                cfgMgr.config.blueBrightness = val;
            }
        }

        if (server.hasArg("disturberBrightness")) {
            long val = String(server.arg("disturberBrightness")).toInt();
            if (val >= 0 && val <= 255) {
                cfgMgr.config.disturberBrightness = val;
            }
        }

        if (server.hasArg("lcdON")) {
            long val = String(server.arg("lcdON")).toInt();
            if (val >= 0 && val <= 23) {
                cfgMgr.config.lcdON = val;
            }
        }

                if (server.hasArg("lcdOFF")) {
            long val = String(server.arg("lcdOFF")).toInt();
            if (val >= 0 && val <= 23) {
                cfgMgr.config.lcdOFF = val;
            }
        }

        cfgMgr.commit();
        handleSettings();

        if (needsClearing) {
            // clear detector statistics after changing the detector config
            detector.clearStatistics();
        }
    }
}

void handleCalibrate() {
    if (authenticated()) {
        unsigned int oldColor = currentColor;
        color(COLOR_CALIBRATING);
        StaticJsonDocument<200> doc;
        doc["tuning"] = detector.calibrate();
        sendJsonResponse(doc);
        color(oldColor);
    }
}

void handleClear() {
    if (authenticated()) {
        detector.clearDetections();
        lastLightningTime = 0;
        detector.strikesTotal = 0;
        b_distance, b_energy, b_intensity = 0;
        server.send(200, "text/plain", "OK");
    }
}

void handleReset() {
    if (authenticated()) {
        detector.reset();
        handleCalibrate();
        setupDetector();
        detector.clearStatistics();
    }
}

void sendMqttStatus() {
    DynamicJsonDocument doc(8192);
    String json;

    Lightning lightning;
    if (detector.getLastLightningDetection(0, lightning)) {
        doc["energy"] = lightning.energy;
        if (lightning.distance < 0x3F) {
            doc["distance"] = lightning.distance;
        } else {
            doc["distance"] = (char*) NULL;
        }
    } else {
        doc["energy"] = (char*) NULL;
        doc["distance"] = (char*) NULL;
    }

    doc["tuning"] = detector.getFrequency();
    doc["noiseFloorLevel"] = detector.getNoiseFloorLevel();
    doc["disturbersPerMinute"] = detector.getDisturbersPerMinute();
    doc["watchdogThreshold"] = detector.getWatchdogThreshold();
    doc["wifiSignalStrength"] = WiFi.RSSI();

    serializeJson(doc, json);
    if (!client.publish(MY_MQTT_TOPIC, json.c_str(), MY_MQTT_RETAIN)) {
        Serial.print("Failed to send MQTT message, rc=");
        Serial.println(client.state());
    }
}

void setupDetector() {
    detector.setOutdoorMode(cfgMgr.config.outdoorMode);
    detector.setWatchdogThreshold(cfgMgr.config.watchdogThreshold);
    detector.setMinimumNumberOfLightning(cfgMgr.config.minimumNumberOfLightning);
    detector.setSpikeRejection(cfgMgr.config.spikeRejection);
}

void color(unsigned int color) {
    if (color != currentColor) {
        neopixel.fill(color);
        neopixel.show();
        currentColor = color;
    }
}

inline int limit(int v) {
    return v <= 255 ? (v >= 0 ? v : 0): 255;
}

void updateColor(unsigned long now) {
    if (!connected) {
        color(now % 1000 <= 500 ? COLOR_CONNECTING : COLOR_BLACK);
        return;
    }

    if (!cfgMgr.config.ledEnabled) {
        color(COLOR_BLACK);
        return;
    }

    // Calculate blue color (noise floor level)
    int blue = 0;
    if (detector.isNoiseFloorLevelOutOfRange()) {
        blue = now % 1000 <= 500 ? (cfgMgr.config.blueBrightness > 128 ? cfgMgr.config.blueBrightness : 128) : 0;
    } else {
        blue = detector.getNoiseFloorLevel() * cfgMgr.config.blueBrightness / (detector.getOutdoorMode() ? 2000 : 146);
    }

    // Calculate lightning age (white) and distance (green to red)
    long red = 0;
    long green = 0;
    long white = 0;
    if (ledLightning.time > 0 && ledLightning.distance <= 40) {
        unsigned long diff = timeDifference(now, ledLightning.time);
        long distancePast = (DISTANCE_ANIMATION_TIME + ENERGY_ANIMATION_TIME) - diff;
        long energyPast = ENERGY_ANIMATION_TIME - diff;
        if (energyPast >= 0) {
            white = ((     ledLightning.energy  ) * energyPast   * 255) / (300000 * ENERGY_ANIMATION_TIME);
            blue = 0;
        } else if (distancePast >= 0) {
            green = ((     ledLightning.distance) * distancePast * 255) / (40 * DISTANCE_ANIMATION_TIME);
            red   = ((40 - ledLightning.distance) * distancePast * 255) / (40 * DISTANCE_ANIMATION_TIME);
            long newblue = ((long) blue) * diff / (DISTANCE_ANIMATION_TIME + ENERGY_ANIMATION_TIME);
            if (newblue < blue) {
                blue = newblue;
            }
        }
    }

    // Calculate last disturber fade
    unsigned long lastDisturber = detector.getLastDisturber();
    unsigned long disturberDiff = timeDifference(now, lastDisturber);
    if (cfgMgr.config.disturberBrightness > 0 && lastDisturber != 0 && disturberDiff <= DISTURBER_ANIMATION_TIME) {
        red = (cfgMgr.config.disturberBrightness * (DISTURBER_ANIMATION_TIME - disturberDiff)) / DISTURBER_ANIMATION_TIME;
    }

    color(    (limit(white) & 0xFF) << 24
            | (limit(red  ) & 0xFF) << 16
            | (limit(green) & 0xFF) << 8
            | (limit(blue ) & 0xFF));
}


// =========================== ODOSLE ZAZNAM O ZACHYTENOM BLESKU NA SERVER ===============================
// nutne spravit cast na sprane servera - prijem dat cez PHP post a zapis napriklad do Mysql databazy
void write_blesk()
{
  String url_blesk;
  // priklad vyskladania URL a odoslanie na server
/*
  HTTPClient http;
  url_blesk = "http://urladresa/zapis.php?";
  url_blesk += "vzdialenost=" + String(b_distance);  
  url_blesk += "&intenzita=" + String(b_intensity);
  url_blesk.replace(" ", "");
  Serial.println(url_blesk);

  bool httpResult = http.begin(wifiClient, url_blesk);
  if(!httpResult)
  {
    Serial.println("Invalid HTTP request:");
    Serial.println(url_blesk);
  }
  else
  {
    int httpCode = http.GET();
    if (httpCode > 0)
    { // Request has been made
      Serial.printf("HTTP status: %d Message: ", httpCode);
      String payload = http.getString();
      Serial.println(payload);
    }
    else
    { // Request could not be made
      Serial.printf("HTTP request failed. Error: %s\r\n", http.errorToString(httpCode).c_str());
    }
  }
  http.end();  
  */
}

// =========================== ODOSLE NAMERANE UDAJE ZO SNIMACOV NA SERVER ===============================
// nutne spravit cast na sprane servera - prijem dat cez PHP post a zapis napriklad do Mysql databazy
void write_snimace()
{
  String url_snimace;
   // priklad vyskladania URL a odoslanie na server
/*  
  HTTPClient http;
  url_snimace = "http://urladresa/zapis.php?";
  url_snimace += "tlak=" + String(iP);  
  url_snimace += "&teplota=" + String(T,1);
  url_snimace += "&vlhkost=" + String(iH);
  url_snimace += "&osvetlenie=" + String(iL);
  url_snimace.replace(" ", "");
  Serial.println(url_snimace);

  bool httpResult = http.begin(wifiClient, url_snimace);
  if(!httpResult)
  {
    Serial.println("Invalid HTTP request:");
    Serial.println(url_snimace);
  }
  else
  {
    int httpCode = http.GET();
    if (httpCode > 0)
    { // Request has been made
      Serial.printf("HTTP status: %d Message: ", httpCode);
      String payload = http.getString();
      Serial.println(payload);
    }
    else
    { // Request could not be made
      Serial.printf("HTTP request failed. Error: %s\r\n", http.errorToString(httpCode).c_str());
    }
  }
  http.end();  
  */
}


// NACITAVA A SPRACUJE DATA ZO SNIMACOV VLHKOSTI, TEPLTOTY, TLAKU
void sensors_getdata()
{
  char status;

  status = pressure.startTemperature();
  if (status != 0)
  {
    delay(status);
    status = pressure.getTemperature(T);
    if (status != 0)
    {            
      status = pressure.startPressure(3);
      if (status != 0)
      {
        delay(status);
        status = pressure.getPressure(P,T);
        p0 =(P/pow(1-(alt/44330.0),5.255));
        if (status != 0)
        {
          p0 = pressure.sealevel(P,alt); 
          a = pressure.altitude(P,p0);
        }
        else Serial.println("error retrieving pressure measurement\n");
      }
      else Serial.println("error starting pressure measurement\n");
    }
    else Serial.println("error retrieving temperature measurement\n");
  }
  else Serial.println("error starting temperature measurement\n");

  if(humidity.measure()) 
     {
        humid = humidity.getHumidity();// read humidity hrom HTU21D
     }

fT = T;
iH = humid;
iP = p0;
iL = light.readLightLevel(); 

veml6075.poll(); // nacita data z UV senzora VEML6075
UVA = veml6075.getUVA();
UVB = veml6075.getUVB();
UVI = veml6075.getUVIndex();

}

// KONIEC


// V NASTAVENU HODINU ZAPNE A VYPNE PODSVIETENIE LCD A UPRAVI SVIT MODREJ LED
void LCDbacklightControl()
{
  if (tm.tm_hour == cfgMgr.config.lcdOFF) 
    {
      lcd.noBacklight();
      cfgMgr.config.blueBrightness = 0;  
    } 
  if (tm.tm_hour == cfgMgr.config.lcdON) 
    {
      lcd.backlight();
      cfgMgr.config.blueBrightness = 48;  
    }
}
// KONIEC
